{%- extends "layout.html" %}
{%- block body %}
<article>
    <h1 class="page-title"><a href="{{ page.cooked_url }}">{{ page.title.strip('\n') }}</a></h1>
    <div class="page-content">
{{ page.content|safe }}
    </div>
</article>
{%- endblock %}
